@@ -14,6 +14,7 @@ import org.gnu.glib.Type;
 import org.gnu.gtk.AccelGroup;
 import org.gnu.gtk.Container;
 import org.gnu.gtk.MenuBar;
+import org.gnu.gtk.MenuShell;
 import org.gnu.gtk.StatusBar;
 import org.gnu.gtk.ToolBar;
 import org.gnu.gtk.VBox;
@@ -50,7 +51,7 @@ public class App extends Window {
 		Handle hndl = getDock(getHandle());
 		if (null == hndl)
 			return null;
-		GObject obj = retrieveGObject(hndl);
+		GObject obj = getGObjectFromHandle(hndl);
 		if (null != obj)
 			return (Widget)obj;
 		return new Widget(hndl);
@@ -70,11 +71,22 @@ public class App extends Window {
 	 */
 	public MenuBar getMenuBar() {
 		Handle hndl = getMenubar(getHandle());
-		GObject obj = retrieveGObject(hndl);
+		GObject obj = getGObjectFromHandle(hndl);
 		if (null != obj)
 			return (MenuBar)obj;
 		return new MenuBar(hndl);
 	}
+	
+	/**
+	 * Returns the contents of the App.
+	 */
+	public Widget getContents() {
+		Handle hndl = getContents(getHandle());
+		GObject obj = getGObjectFromHandle(hndl);
+		if (null != obj)
+			return (Widget)obj;
+		return new Widget(hndl);
+	}
 
 	/**
 	 * Gets the name of the application.
@@ -98,7 +110,7 @@ public class App extends Window {
 	 */
 	public StatusBar getStatusBar() {
 		Handle hndl = getStatusbar(getHandle());
-		GObject obj = retrieveGObject(hndl);
+		GObject obj = getGObjectFromHandle(hndl);
 		if (null != obj)
 			return (StatusBar)obj;
 		return new StatusBar(hndl);
@@ -110,7 +122,7 @@ public class App extends Window {
 	 */
 	public VBox getVBox() {
 		Handle hndl = getVbox(getHandle());
-		GObject obj = retrieveGObject(hndl);
+		GObject obj = getGObjectFromHandle(hndl);
 		if (null != obj)
 			return (VBox)obj;
 		return new VBox(hndl);
@@ -129,8 +141,8 @@ public class App extends Window {
 	/**
 	 * If true, the application uses gnome-config to retrieve and save the
 	 * docking configuration automatically.
-	 * @param enable True if the docking configuration is to be handled
-	 *               automatically, else false.
+	 * @param enabled True if the docking configuration is to be handled
+	 *                automatically, else false.
 	 */
 	public void layoutConfig(boolean enabled) {
 		gnome_app_enable_layout_config(getHandle(), enabled);
@@ -289,7 +301,53 @@ public class App extends Window {
 		gnome_app_install_menu_hints(getHandle(), hndls);
 	}
 
+    /**
+     * Fills the specified MenuShell with items created from the UIInfo array, 
+     * inserting them from the item number <i>position</i> on.  The AccelGroup
+     * will be used for all newly created sub menus and servers as the global
+     * AccelGroup for all menu item hotkeys.
+     * 
+     * @param menuShell
+     * @param uiinfos
+     * @param accelGroup
+     * @param underline
+     * @param position
+     */
+	public void fillMenu(MenuShell menuShell, UIInfo[] uiinfos, AccelGroup accelGroup, boolean underline, int position) {
+        Handle[] hndls = new Handle[uiinfos.length];
+        for (int i = 0; i < uiinfos.length; i++)
+            hndls[i] = uiinfos[i].getHandle();
+	    gnome_app_fill_menu(menuShell.getHandle(), hndls, accelGroup.getHandle(), underline, position);
+    }
+    
 	/**
+     * Fills a ToolBar with the specified UIInfos. 
+     * @param toolbar
+     * @param uiinfos
+     * @param accelGroup
+	 */
+    public void fillToolbar(ToolBar toolbar, UIInfo[] uiinfos, AccelGroup accelGroup) {
+        Handle[] hndls = new Handle[uiinfos.length];
+        for (int i = 0; i < uiinfos.length; i++)
+            hndls[i] = uiinfos[i].getHandle();
+        gnome_app_fill_toolbar(toolbar.getHandle(), hndls, accelGroup.getHandle());
+    }
+
+    /**
+     * Removes items from the Menu.
+     * @param path
+     * @param start
+     * @param numItems
+     */
+    public void removeMenuRange(String path, int start, int numItems) {
+        gnome_app_remove_menu_range(getHandle(), path, start, numItems);
+    }
+
+    
+    
+    
+    
+    /**
 	 * Display a simple message in an OK dialog or the status bar.  Requires
 	 * confirmation from the user before it goes away.
 	 * @param message The text to display.
@@ -300,7 +358,7 @@ public class App extends Window {
 		Handle hndl = gnome_app_message(getHandle(), message);
 		if (null == hndl)
 			return null;
-		GObject obj = retrieveGObject(hndl);
+		GObject obj = getGObjectFromHandle(hndl);
 		if (null != obj)
 			return (Widget)obj;
 		return new Widget(hndl);
@@ -326,7 +384,7 @@ public class App extends Window {
 		Handle hndl = gnome_app_warning(getHandle(), warning);
 		if (null == hndl)
 			return null;
-		GObject obj = retrieveGObject(hndl);
+		GObject obj = getGObjectFromHandle(hndl);
 		if (null != obj)
 			return (Widget)obj;
 		return new Widget(hndl);
@@ -342,7 +400,7 @@ public class App extends Window {
 		Handle hndl = gnome_app_error(getHandle(), error);
 		if (null == hndl)
 			return null;
-		GObject obj = retrieveGObject(hndl);
+		GObject obj = getGObjectFromHandle(hndl);
 		if (null != obj)
 			return (Widget)obj;
 		return new Widget(hndl);
@@ -373,11 +431,17 @@ public class App extends Window {
 	native static final protected void gnome_app_set_statusbar_custom(Handle app, Handle container, Handle statusbar);
 	native static final protected void gnome_app_set_contents(Handle app, Handle contents);
 	native static final protected void gnome_app_enable_layout_config(Handle app, boolean enable);
+	native static final protected void gnome_app_fill_menu(Handle menuShell, Handle[] uiInfo, Handle accelGroup, boolean ulineAccels, int pos);
+	native static final protected void gnome_app_fill_menu_with_data(Handle menuShell, Handle[] uiInfo, Handle accelGroup, boolean ulineAccels, int pos, Object userData);
+	native static final protected void gnome_app_fill_menu_custom(Handle menuShell, Handle[] uiInfo, Handle uiBData, Handle accelGroup, boolean ulineAccels, int pos);
 	native static final protected void gnome_app_ui_configure_configurable(Handle uiinfo);
 	native static final protected void gnome_app_create_menus(Handle app, Handle[] uiinfo);
-	native static final protected void gnome_app_create_menus_custom(Handle app, Handle uiinfo, Handle uibdate);
+	native static final protected void gnome_app_create_menus_custom(Handle app, Handle[] uiinfo, Handle uibdate);
 	native static final protected void gnome_app_create_toolbar(Handle app, Handle[] uiinfo);
-	native static final protected void gnome_app_create_toolbar_custom(Handle app, Handle uiinfo, Handle uibdata);
+	native static final protected void gnome_app_create_toolbar_custom(Handle app, Handle[] uiinfo, Handle uibdata);
+	native static final protected void gnome_app_fill_toolbar(Handle toolbar, Handle[] uiInfo, Handle accelGroup);
+	native static final protected void gnome_app_fill_toolbar_with_data(Handle toolbar, Handle[] uiInfo, Handle accelGroup, Object userData);
+	native static final protected void gnome_app_fill_toolbar_custom(Handle toolbar, Handle[] uiInfo, int uiBData, Handle accelGroup);
 	native static final protected int gnome_app_find_menu_pos(Handle parent, String path, int pos);
 	native static final protected void gnome_app_remove_menus(Handle app, String path, int items);
 	native static final protected void gnome_app_remove_menu_range(Handle app, String path, int start, int items);
@@ -386,15 +450,10 @@ public class App extends Window {
 	native static final protected void gnome_app_install_appbar_menu_hints(Handle appbar, Handle[] uiinfo);
 	native static final protected void gnome_app_install_statusbar_menu_hints(Handle bar, Handle[] uiinfo);
 	native static final protected void gnome_app_install_menu_hints(Handle app, Handle[] uiinfo);
-	native static final protected void gnome_app_fill_menu(Handle menuShell, Handle uiInfo, Handle accelGroup, boolean ulineAccels, int pos);
-	native static final protected void gnome_app_fill_menu_with_data(Handle menuShell, Handle uiInfo, Handle accelGroup, boolean ulineAccels, int pos, Object userData);
-	native static final protected void gnome_app_fill_menu_custom(Handle menuShell, Handle uiInfo, Handle uiBData, Handle accelGroup, boolean ulineAccels, int pos);
-	native static final protected void gnome_app_fill_toolbar(Handle toolbar, Handle uiInfo, Handle accelGroup);
-	native static final protected void gnome_app_fill_toolbar_with_data(Handle toolbar, Handle uiInfo, Handle accelGroup, Object userData);
-	native static final protected void gnome_app_fill_toolbar_custom(Handle toolbar, Handle uiInfo, int uiBData, Handle accelGroup);
     native static final protected Handle gnome_app_message (Handle app, String message);
     native static final protected void gnome_app_flash (Handle app, String flash);
     native static final protected Handle gnome_app_error (Handle app, String error);
     native static final protected Handle gnome_app_warning (Handle app, String warning);
 
 }
+
